package sun.misc;

import java.security.PrivilegedAction;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.nio.ByteBuffer;

public final class Perf
{
    private static Perf instance;
    private static final int PERF_MODE_RO = 0;
    private static final int PERF_MODE_RW = 1;
    
    private Perf() {
    }
    
    public static Perf getPerf() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("sun.misc.Perf.getPerf"));
        }
        return Perf.instance;
    }
    
    public ByteBuffer attach(int n, String s) throws IllegalArgumentException, IOException {
        if (s.compareTo("r") == 0) {
            return this.attachImpl(null, n, 0);
        }
        if (s.compareTo("rw") == 0) {
            return this.attachImpl(null, n, 1);
        }
        throw new IllegalArgumentException("unknown mode");
    }
    
    public ByteBuffer attach(String s, int n, String s2) throws IllegalArgumentException, IOException {
        if (s2.compareTo("r") == 0) {
            return this.attachImpl(s, n, 0);
        }
        if (s2.compareTo("rw") == 0) {
            return this.attachImpl(s, n, 1);
        }
        throw new IllegalArgumentException("unknown mode");
    }
    
    private ByteBuffer attachImpl(String s, int n, int n2) throws IllegalArgumentException, IOException {
        final ByteBuffer attach = this.attach(s, n, n2);
        if (n == 0) {
            return attach;
        }
        ByteBuffer duplicate = attach.duplicate();
        Cleaner.create(duplicate, new Runnable() {
            @Override
            public void run() {
                try {
                    Perf.access$100(Perf.instance, attach);
                }
                catch (Throwable t) {
                    assert false : t.toString();
                }
            }
        });
        return duplicate;
    }
    
    private native ByteBuffer attach(String p0, int p1, int p2) throws IllegalArgumentException, IOException;
    
    private native void detach(ByteBuffer p0);
    
    public native ByteBuffer createLong(String p0, int p1, int p2, long p3);
    
    public ByteBuffer createString(String s, int n, int n2, String s2, int b) {
        byte[] bytes = getBytes(s2);
        byte[] array = new byte[bytes.length + 1];
        System.arraycopy(bytes, 0, array, 0, bytes.length);
        array[bytes.length] = 0;
        return this.createByteArray(s, n, n2, array, Math.max(array.length, b));
    }
    
    public ByteBuffer createString(String s, int n, int n2, String s2) {
        byte[] bytes = getBytes(s2);
        byte[] array = new byte[bytes.length + 1];
        System.arraycopy(bytes, 0, array, 0, bytes.length);
        array[bytes.length] = 0;
        return this.createByteArray(s, n, n2, array, array.length);
    }
    
    public native ByteBuffer createByteArray(String p0, int p1, int p2, byte[] p3, int p4);
    
    private static byte[] getBytes(String s) {
        byte[] bytes = null;
        try {
            bytes = s.getBytes("UTF-8");
        }
        catch (UnsupportedEncodingException ex) {}
        return bytes;
    }
    
    public native long highResCounter();
    
    public native long highResFrequency();
    
    private static native void registerNatives();
    
    static void access$100(Perf perf, ByteBuffer byteBuffer) {
        perf.detach(byteBuffer);
    }
    
    static {
        registerNatives();
        Perf.instance = new Perf();
    }
    
    public static class GetPerfAction implements PrivilegedAction<Perf>
    {
        @Override
        public Perf run() {
            return Perf.getPerf();
        }
    }
}
